#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include "stop-list.h"
#include "util.h"
#include "idxname.h"
#include "namazu.h"

static char **stop_list;
static int error=0;

void clear_word_list(void)
{
	int i;

	for (i = 0; stop_list[i]; i++) {
	/* last element of stop_stop_list is NULL */
		free(stop_list[i]);
		stop_list[i] = NULL;
	}

	free(stop_list);

	stop_list = NULL;
}

int read_stop_list(void)
{
	char word[MAX_STOP_LENGTH + 1] = "";
	char stop_file[BUFSIZE + 20] = "stopwords.txt";
	int fd, i = 0, j = 0;
	char c;

	char *base;
	char *templatedir = get_templatedir();

	if (templatedir[0] != '\0') {
		base = templatedir;
	} else if (nmz_get_idxnum() == 1) {
		base = nmz_get_idxname(0);
	} else {
		base = nmz_get_defaultidx();
	}

	nmz_pathcat(base, stop_file);

	if((fd = open(stop_file, O_RDONLY)) == -1) {
		error = -1;
		return (int)FAILURE;
	}

	stop_list = (char **) calloc(MAX_STOP + 1, sizeof(char *));

	while (read(fd, &c, 1)) {
		if (c == '\n' && strlen(word)) {
			/* Ok, so it's the end of the word, and I need to
			 * transfer it into the stop_list.
			 * malloc enough memory for the word and insert it
			 * into the ith position of stop_list
			 */
			stop_list[i] =
			    (char *) calloc(1 + strlen(word),
					    sizeof(char));
			strcpy(stop_list[i++], word);
			stop_list[i] = NULL;	/* make sure last element is
						   always null */

			j = 0;
			word[j] = 0;

			if (i == MAX_STOP - 1) {
/*				do_logging ("Too many stop words. Max", MAX_STOP); */
				break;
			}
		} else {
			if (j >= MAX_STOP_LENGTH - 1) {
/*				do_logging ("Stop word too long. Max", MAX_STOP_LENGTH); */
				continue;
			}
			/* Will only come here if j is less than 
			 * MAX_STOP_LENGTH-1
			 */
			word[j++] = c;
			word[j] = '\0';
		}
	}
	close(fd);

	/* If the last word does not end with a newline, then it will
	 * still be in word.  Add it now.
	 */
	if (strlen(word)) {
		stop_list[i] =
		    (char *) malloc(1 + strlen(word) * sizeof(char));
		strcpy(stop_list[i], word);
	}

	return SUCCESS;
}

char **parse_words(char *string)
{
	char **word_list;
	int i = 0, j, k;
	int words = 1, quote = 0, word_start = 0, length = 0;

	for (j = 0; string[j]; j++) {
		switch (string[j]) {
		case '"':
			quote = !quote;
			break;
		case ' ':
			if (!quote)
				words++;
			break;
		}
	}
	word_list = (char **) calloc(words + 1, sizeof(char *));

	for (j = 0; j == 0 || string[j - 1]; j++) {
		switch (string[j]) {
		case '"':
			quote = !quote;
			length++;
			break;

		case '\0':
		case ' ':
			if (!quote) {
				/* This has to be done before writing
				 * the array, to avoid problems with
				 * multiple spaces
				 */
				k = word_start;
				word_start = j + 1;	/* reset word_start to 
							   next word */

				if (length == 0)
					break;

				word_list[i] =
				    (char *) calloc(length + 1, sizeof(char));

				strncpy(word_list[i], string + k, length);
				word_list[i++][length] = '\0';

				length = 0;
				word_list[i] = NULL;
			} else {
				length++;
			}
			break;

		default:
			length++;
			break;
		}
	}

	return word_list;
}

int is_stop_word(char * word)
{
	int i;

	if(error)
		return 0;

	for (i = 0; stop_list[i]; i++)	/* stop_list ends with NULL */
		if (!strcasecmp(word, stop_list[i]))
			return 1;
	return 0;
}

char *strip_stop_words(char *query)
{
	char **word_list;
	int i, j, match = 0;

	if (!strlen(query))
		return query;

	word_list = parse_words(query);
	if (!word_list[0])
		return query;

	read_stop_list();

	for (i = 0; word_list[i]; i++) {	/* word_list ends with NULL */
		match = 0;

		if (strlen(word_list[i]) < 2) {
			strcpy(word_list[i], " ");
			continue;
		}

		if (is_stop_word(word_list[i])) {
			strcpy(word_list[i], " ");
			continue;
		}
	}

	clear_word_list();

	strcpy(query, "");
	for (i = 0; word_list[i]; i++)
		if (strcmp(word_list[i], " ")) {
			strcat(query, word_list[i]);
			strcat(query, " ");
		}


	for (i = 0; word_list[i]; i++) {
		free(word_list[i]);
		word_list[i] = NULL;
	}
	free(word_list);
	word_list = NULL;

	return query;
}

